@@ -2,6 +2,7 @@ import * as path from 'path';
 import createFilter from 'rollup-pluginutils/src/createFilter';
 import { rollupInternal } from '../rollup/rollup';
 import {
+	ChangeEvent,
 	MergedRollupOptions,
 	OutputOptions,
 	RollupBuild,
@@ -13,12 +14,30 @@ import { mergeOptions } from '../utils/options/mergeOptions';
 import { GenericConfigObject } from '../utils/options/options';
 import { FileWatcher } from './fileWatcher';
 
+const eventsRewrites: Record<ChangeEvent, Record<ChangeEvent, ChangeEvent | 'buggy' | null>> = {
+	create: {
+		create: 'buggy',
+		delete: null,			//delete file from map
+		update: 'create',
+	},
+	delete: {
+		create: 'update',
+		delete: 'buggy',
+		update: 'buggy',
+	},
+	update: {
+		create: 'buggy',
+		delete: 'delete',
+		update: 'update',
+	}
+}
+
 export class Watcher {
 	emitter: RollupWatcher;
 
 	private buildDelay = 0;
 	private buildTimeout: NodeJS.Timer | null = null;
-	private invalidatedIds: Set<string> = new Set();
+	private invalidatedIds: Map<string, ChangeEvent> = new Map();
 	private rerun = false;
 	private running: boolean;
 	private tasks: Task[];
@@ -43,16 +62,25 @@ export class Watcher {
 		for (const task of this.tasks) {
 			task.close();
 		}
+		this.emitter.emit('close');
 		this.emitter.removeAllListeners();
 	}
 
-	emit(event: string, value?: any) {
-		this.emitter.emit(event as any, value);
-	}
+	invalidate(file?: {event: ChangeEvent, id: string}) {
+		if (file) {
+			const prevEvent = this.invalidatedIds.get(file.id);
+			const event = prevEvent
+				? eventsRewrites[prevEvent][file.event]
+				: file.event;
 
-	invalidate(id?: string) {
-		if (id) {
-			this.invalidatedIds.add(id);
+			if (event === 'buggy') {
+				//TODO: throws or warn? Currently just ignore, uses new event
+				this.invalidatedIds.set(file.id, file.event);
+			} else if (event === null) {
+				this.invalidatedIds.delete(file.id);
+			} else {
+				this.invalidatedIds.set(file.id, event);
+			}
 		}
 		if (this.running) {
 			this.rerun = true;
@@ -63,11 +91,11 @@ export class Watcher {
 
 		this.buildTimeout = setTimeout(() => {
 			this.buildTimeout = null;
-			for (const id of this.invalidatedIds) {
-				this.emit('change', id);
+			for (const [id, event] of this.invalidatedIds.entries()) {
+				this.emitter.emit('change', id, {event});
 			}
 			this.invalidatedIds.clear();
-			this.emit('restart');
+			this.emitter.emit('restart');
 			this.run();
 		}, this.buildDelay);
 	}
@@ -75,7 +103,7 @@ export class Watcher {
 	private async run() {
 		this.running = true;
 
-		this.emit('event', {
+		this.emitter.emit('event', {
 			code: 'START'
 		});
 
@@ -84,12 +112,12 @@ export class Watcher {
 				await task.run();
 			}
 			this.running = false;
-			this.emit('event', {
+			this.emitter.emit('event', {
 				code: 'END'
 			});
 		} catch (error) {
 			this.running = false;
-			this.emit('event', {
+			this.emitter.emit('event', {
 				code: 'ERROR',
 				error
 			});
@@ -144,16 +172,16 @@ export class Task {
 		this.fileWatcher.close();
 	}
 
-	invalidate(id: string, isTransformDependency: boolean | undefined) {
+	invalidate(id: string, details: {event: ChangeEvent, isTransformDependency?: boolean}) {
 		this.invalidated = true;
-		if (isTransformDependency) {
+		if (details.isTransformDependency) {
 			for (const module of this.cache.modules) {
 				if (module.transformDependencies.indexOf(id) === -1) continue;
 				// effective invalidation
 				module.originalCode = null as any;
 			}
 		}
-		this.watcher.invalidate(id);
+		this.watcher.invalidate({id, event: details.event});
 	}
 
 	async run() {
@@ -167,7 +195,7 @@ export class Task {
 
 		const start = Date.now();
 
-		this.watcher.emit('event', {
+		this.watcher.emitter.emit('event', {
 			code: 'BUNDLE_START',
 			input: this.options.input,
 			output: this.outputFiles
@@ -180,7 +208,7 @@ export class Task {
 			}
 			this.updateWatchedFiles(result);
 			this.skipWrite || (await Promise.all(this.outputs.map(output => result.write(output))));
-			this.watcher.emit('event', {
+			this.watcher.emitter.emit('event', {
 				code: 'BUNDLE_END',
 				duration: Date.now() - start,
 				input: this.options.input,
