@@ -1,11 +1,11 @@
 (function (factory) {
 	typeof define === 'function' && define.amd ? define(factory) :
 	factory();
-}(function () { 'use strict';
+}((function () { 'use strict';
 
 	function baz() {
 		console.log("baz");
 	}
 	baz();
 
-}));
+})));
