@@ -7,7 +7,18 @@ module.exports = {
 		code: 'PARSE_ERROR',
 		message:
 			'Unexpected token (Note that you need plugins to import files that are not JavaScript)',
+		id: path.resolve(__dirname, 'file.css'),
+		parserError: {
+			loc: {
+				column: 0,
+				line: 1
+			},
+			message: 'Unexpected token (1:0)',
+			pos: 0,
+			raisedAt: 1
+		},
 		pos: 0,
+		watchFiles: [path.resolve(__dirname, 'main.js'), path.resolve(__dirname, 'file.css')],
 		loc: {
 			file: path.resolve(__dirname, 'file.css'),
 			line: 1,
